
import math
import numpy as np
import matplotlib.pylab as plt
from numpy import array, arange
from scipy.integrate import odeint
import time

########################################################

"""IMPLEMENTACJA METODY RK2 MIDPOINT"""

def RK2(f,x0,t0,tk,h): 
    t=np.arange(t0,tk,h)
    N=len(t)
    if hasattr(x0, "__len__"): x=np.zeros((N,len(x0)))
    else: x=np.zeros(N)
    x[0]=np.array(x0)
    i=1
    while (i<N):
        k1=h*f(x[i-1], t[i-1])
        k2=h*f(x[i-1]+0.5*k1, t[i-1]+0.5*h)
        x[i]=np.array(x[i-1]+k2)
        i+=1
    return t,x


########################################################

"""RÓWNANIE RUCHU ORAZ ROZBICIE GO NA CZĘSCI"""

def f(y,t):
    k = 3.94
    p = 2
    m = 0.1
    
    x = y[0]
    v = y[1]
    dxdt = v
    dvdt = -k*x**(p-1)/m #+ 5*math.sin(6.28*x) 
    return np.array([dxdt, dvdt])

########################################################

"""ROZWIĄZANIE METODĄ ODEINT DLA PORÓWNANIA"""

trange = 5
h = 0.01

ts = arange(0, trange, h)
x0 = 0.0
v0 = 6.28
y0 = array([x0, v0])

ys = odeint(f, y0, ts)
xs=ys[:,0]
vs=ys[:,1]

########################################################

"""WYZNACZENIE BŁĘDU OBLICZEŃ METODY RK2 MIDPOINT"""

start = time.time()
rk2 = RK2(f,[0.0,6.28],0,trange,h)
end = time.time()

#print("Lista wartosci obliczonych metodą RK2 MidPoint: ", RK2(f,[0.0,1.0],0,trange,h)[1])
#print("Lista dokładnych wartosci: ", ys)
yerr = rk2[1]-ys
#print("Tablica błędów wartosci w każdym punkcie: ", yerr)

lista1 = []
lista2 = []
for u in yerr:
    lista1.append(u[0])
    lista2.append(u[1])

lista1 = np.array(lista1)
lista2 = np.array(lista2)

errorXvec = math.sqrt(np.sum( lista1**2 ))
errorXavg = np.sum(abs(lista1))/lista1.shape[0]

errorVvec = math.sqrt(np.sum( lista2**2 ))
errorVavg = np.sum(abs(lista2))/lista2.shape[0]

########################################################

"""WYPISANIE INFORMACJI ORAZ RYSOWANIE WYKRESÓW"""

print("-------------------------------------")
print("OSCYLATOR HARMONICZNY:")
print("Błąd położenia x: ", errorXavg)
print("Błąd prędkosci v: ", errorVavg)
print('Czas obliczenia metodą RK2 MidPoint dla zakresu [0,'+ str(trange) +']: ',end-start, 's')
print("-------------------------------------")

plt.plot(ts,ys)
plt.plot(*rk2)
plt.show()



########################################################
########################################################

"""RÓWNANIE (1) 2yy" + 2y^2 - 2y'^2 = 0 ORAZ ROZBICIE GO NA CZĘSCI"""

trange1 = 20
h1 = 0.01

def r1(y,t):
    ynormal = y[0]
    yprim = y[1]
    dydt = yprim
    dyprimdt = (dydt**2-2*ynormal**2)/ynormal
    return np.array([dydt,dyprimdt])

########################################################

"""ROZWIĄZANIE RÓWNANIA(1) METODĄ ODEINT DLA PORÓWNANIA"""

ts1 = arange(0, trange1, h1)
ynormal0 = 1.0
yprim0 = 1.0
y0a = array([ynormal0, yprim0])

ys1 = odeint(r1, y0a, ts1)
xs1=ys1[:,0]
vs1=ys1[:,1]

########################################################

"""WYZNACZENIE BŁĘDU OBLICZEŃ METODY RK2 MIDPOINT DLA RÓWNANIA (1)"""

start1 = time.time()
rk2a = RK2(r1,[1.0,1.0],0,trange1,h1)
end1 = time.time()

#print("Lista wartosci obliczonych metodą RK2 MidPoint: ", RK2(r1,[1.0,1.0],0,trange1,h1)[1])
#print("Lista dokładnych wartosci: ", ys1)
yerr1 = rk2a[1]-ys1
#print("Tablica błędów wartosci w każdym punkcie: ", yerr1)

lista1a = []
lista2a = []
for u in yerr1:
    lista1a.append(u[0])
    lista2a.append(u[1])

lista1a = np.array(lista1a)
lista2a = np.array(lista2a)

errorYnormalvec = math.sqrt(np.sum( lista1a**2 ))
errorYnormalavg = np.sum(abs(lista1a))/lista1a.shape[0]

errorYprimvec = math.sqrt(np.sum( lista2a**2 ))
errorYprimavg = np.sum(abs(lista2a))/lista2a.shape[0]

########################################################

"""WYPISANIE INFORMACJI ORAZ RYSOWANIE WYKRESÓW"""
print("-------------------------------------")
print("RÓWNANIE (1)")
print("Błąd ynormal: ", errorYnormalavg)
print("Błąd yprim: ", errorYprimavg)
print('Czas obliczenia (1) metodą RK2 MidPoint dla zakresu [0,'+ str(trange1) +']: ',end1-start1, 's')
print("-------------------------------------")

plt.plot(ts1,ys1)
plt.plot(*rk2a)
plt.show()

########################################################
########################################################

"""RÓWNANIE (2) y" + 6y^5 = 0 ORAZ ROZBICIE GO NA CZĘSCI"""

trange2 = 20
h2 = 0.01

def r2(y,t):
    ynormal1 = y[0]
    yprim1 = y[1]
    dydt1 = yprim1
    dyprimdt1 = -6*ynormal1**5
    return np.array([dydt1,dyprimdt1])

########################################################

"""ROZWIĄZANIE RÓWNANIA (2) METODĄ ODEINT DLA PORÓWNANIA"""

ts2 = arange(0, trange2, h1)
ynormal0a = 1.0
yprim0a = 1.0
y0b = array([ynormal0a, yprim0a])

ys2 = odeint(r2, y0b, ts2)
xs2=ys2[:,0]
vs2=ys2[:,1]

########################################################

"""WYZNACZENIE BŁĘDU OBLICZEŃ METODY RK2 MIDPOINT DLA RÓWNANIA (2)"""

start2 = time.time()
rk2b = RK2(r2,[1.0,1.0],0,trange2,h2)
end2 = time.time()

#print("Lista wartosci obliczonych metodą RK2 MidPoint: ", RK2(r2,[1.0,1.0],0,trange2,h2)[1])
#print("Lista dokładnych wartosci: ", ys2)
yerr2 = rk2b[1]-ys2
#print("Tablica błędów wartosci w każdym punkcie: ", yerr1)

lista1b = []
lista2b = []
for u in yerr2:
    lista1b.append(u[0])
    lista2b.append(u[1])

lista1b = np.array(lista1b)
lista2b = np.array(lista2b)

errorYnormalvec1 = math.sqrt(np.sum( lista1b**2 ))
errorYnormalavg1 = np.sum(abs(lista1b))/lista1b.shape[0]

errorYprimvec1 = math.sqrt(np.sum( lista2b**2 ))
errorYprimavg1 = np.sum(abs(lista2b))/lista2b.shape[0]

########################################################

"""WYPISANIE INFORMACJI ORAZ RYSOWANIE WYKRESÓW"""
print("-------------------------------------")
print("RÓWNANIE (2)")
print("Błąd ynormal: ", errorYnormalavg1)
print("Błąd yprim: ", errorYprimavg1)
print('Czas obliczenia (2) metodą RK2 MidPoint dla zakresu [0,'+ str(trange2) +']: ',end2-start2, 's')
print("-------------------------------------")

plt.plot(ts2,ys2)
plt.plot(*rk2b)
plt.show()