import math
import numpy as np
import matplotlib.pylab as plt
from numpy import array, arange
from scipy.integrate import odeint
import time

def rkf( f, a, b, x0, tol, hmax, hmin ):

    a2  =   2.500000000000000e-01  #  1/4
    a3  =   3.750000000000000e-01  #  3/8
    a4  =   9.230769230769231e-01  #  12/13
    a5  =   1.000000000000000e+00  #  1
    a6  =   5.000000000000000e-01  #  1/2

    b21 =   2.500000000000000e-01  #  1/4
    b31 =   9.375000000000000e-02  #  3/32
    b32 =   2.812500000000000e-01  #  9/32
    b41 =   8.793809740555303e-01  #  1932/2197
    b42 =  -3.277196176604461e+00  # -7200/2197
    b43 =   3.320892125625853e+00  #  7296/2197
    b51 =   2.032407407407407e+00  #  439/216
    b52 =  -8.000000000000000e+00  # -8
    b53 =   7.173489278752436e+00  #  3680/513
    b54 =  -2.058966861598441e-01  # -845/4104
    b61 =  -2.962962962962963e-01  # -8/27
    b62 =   2.000000000000000e+00  #  2
    b63 =  -1.381676413255361e+00  # -3544/2565
    b64 =   4.529727095516569e-01  #  1859/4104
    b65 =  -2.750000000000000e-01  # -11/40

    r1  =   2.777777777777778e-03  #  1/360
    r3  =  -2.994152046783626e-02  # -128/4275
    r4  =  -2.919989367357789e-02  # -2197/75240
    r5  =   2.000000000000000e-02  #  1/50
    r6  =   3.636363636363636e-02  #  2/55

    c1  =   1.157407407407407e-01  #  25/216
    c3  =   5.489278752436647e-01  #  1408/2565
    c4  =   5.353313840155945e-01  #  2197/4104
    c5  =  -2.000000000000000e-01  # -1/5

    t = a
    x = np.array(x0)
    h = hmax

    T = np.array( [t] )
    X = np.array( [x] )
    R = []
    H = []
    
    while t < b:

        if t + h > b:
            h = b - t

        k1 = h * f( x, t )
        k2 = h * f( x + b21 * k1, t + a2 * h )
        k3 = h * f( x + b31 * k1 + b32 * k2, t + a3 * h )
        k4 = h * f( x + b41 * k1 + b42 * k2 + b43 * k3, t + a4 * h )
        k5 = h * f( x + b51 * k1 + b52 * k2 + b53 * k3 + b54 * k4, t + a5 * h )
        k6 = h * f( x + b61 * k1 + b62 * k2 + b63 * k3 + b64 * k4 + b65 * k5, \
                    t + a6 * h )

        r = abs( r1 * k1 + r3 * k3 + r4 * k4 + r5 * k5 + r6 * k6 ) / h
        if len( np.shape( r ) ) > 0:
            r = max( r )
        if r <= tol:
            t = t + h
            x = x + c1 * k1 + c3 * k3 + c4 * k4 + c5 * k5
            T = np.append( T, t )
            X = np.append( X, [x], 0 )

        h = h * min( max( 0.84 * ( tol / r )**0.25, 0.1 ), 4.0 )
        

        if h > hmax:
            h = hmax
        elif h < hmin:
            raise RuntimeError("Error: Could not converge to the required tolerance %e with minimum stepsize  %e." % (tol,hmin))
            break
        
        R.append(r)
        H.append(h)
        
    return ( T, X, R, H)


def f(y,t):
    k = 3.94
    p = 2
    m = 0.1
    
    x = y[0]
    v = y[1]
    dxdt = v
    dvdt = -k*x**(p-1)/m #+ 5*math.sin(6.28*x) 
    return np.array([dxdt, dvdt])

def r1(y,t):
    ynormal = y[0]
    yprim = y[1]
    dydt = yprim
    dyprimdt = (dydt**2-2*ynormal**2)/ynormal
    return np.array([dydt,dyprimdt])

def r2(y,t):
    ynormal1 = y[0]
    yprim1 = y[1]
    dydt1 = yprim1
    dyprimdt1 = -6*ynormal1**5
    return np.array([dydt1,dyprimdt1])

start = time.time()
T, X, r, h = rkf(f, a=0, b=5, x0=[0.0,6.28], tol=1e-6, hmax=1e1, hmin=1e-16)
end = time.time()

start1 = time.time()
T_r1, X_r1, r_r1, h_r1 = rkf(r1, a=0, b=15, x0=[1.0,1.0], tol=1e-6, hmax=1e1, hmin=1e-16)
end1 = time.time()

start2 = time.time()
T_r2, X_r2, r_r2, h_r2 = rkf(r2, a=0, b=15, x0=[1.0,1.0], tol=1e-6, hmax=1e1, hmin=1e-16)
end2 = time.time()

r = np.array(r)
r_r1 = np.array(r_r1)
r_r2 = np.array(r_r2)
h = np.array(h)
h_r1 = np.array(h_r1)
h_r2 = np.array(h_r2)

index = [0, 1]
new_r = np.delete(r, index)
new_r1 = np.delete(r_r1, index)
new_r2 = np.delete(r_r2, index)
new_h = np.delete(h, index)
new_h1 = np.delete(h_r1, index)
new_h2 = np.delete(h_r2, index)

avg_r = np.sum(abs(new_r))/new_r.shape[0]
avg_r1 = np.sum(abs(new_r1))/new_r1.shape[0]
avg_r2 = np.sum(abs(new_r2))/new_r2.shape[0]
avg_h = np.sum(abs(new_h))/new_h.shape[0]
avg_h1 = np.sum(abs(new_h1))/new_h1.shape[0]
avg_h2 = np.sum(abs(new_h2))/new_h2.shape[0]


#print(T, X)
#print(T_r1, X_r1)
#print(T_r2, X_r2)
#print(avg_r, avg_r1, avg_r2)
#print(avg_h, avg_h1, avg_h2)

plt.plot(T, X)
plt.show()
plt.plot(T_r1, X_r1)
plt.show()
plt.plot(T_r2, X_r2)
plt.show()

print("-------------------------------------")
print("OSCYLATOR HARMONICZNY:")
print("Średni błąd: ", avg_r)
print("Średni krok: ", avg_h)
print("Czas obliczenia metodą RK45: ", end-start, "s" )
print("-------------------------------------")
print("-------------------------------------")
print("RÓWNANIE (1):")
print("Średni błąd: ", avg_r1)
print("Średni krok: ", avg_h1)
print("Czas obliczenia metodą RK45: ", end1-start1, "s" )
print("-------------------------------------")
print("-------------------------------------")
print("RÓWNANIE (2):")
print("Średni błąd: ", avg_r2)
print("Średni krok: ", avg_h2)
print("Czas obliczenia metodą RK45: ", end2-start2, "s" )
print("-------------------------------------")
